#========================================================================================
# package      DE_EPAGES::AppStore::API::TLE::TLEHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This class contains common methods for TLE functions.
#========================================================================================
package DE_EPAGES::AppStore::API::TLE::AppStoreHandler;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::AppStore::API::Tools;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);
use DE_EPAGES::AppStore::API::Shop qw( GetUnityScriptTagsApp );
use DE_EPAGES::Presentation::API::Session qw (
    GetSessionAttribute
    SetSessionAttribute
);
use JSON;
use strict;


#---------------------------------------------------------------------------
# TLE Registration
#---------------------------------------------------------------------------
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'}, "IS_APP_AVAILABLE" );
    __PACKAGE__->new()->register( $Params->{'Processor'}, "APP_ID" );
    __PACKAGE__->new()->register( $Params->{'Processor'}, "GetUnityScriptTags" );

    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# input        %args | args | array
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;
    my $self  = bless \%args, $class;

    return $self;
}

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor, $TLEFunctionName );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor       | tle processor     | object
# input        $TLEFunctionName | TLE Function Name | string
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor, $TLEFunctionName) = @_;

    GetLog->debug( "$TLEFunctionName.register" );
    $Processor->registerHandler( 'FunctionHandler', $self, $TLEFunctionName );

    return;
}

#========================================================================================
# function     GetUnityScriptTags
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hInfo = GetUnityScriptTags( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# description  Returns a hash information of this shop's script tags, along
#               with other bits of useful information
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       $hInfo | Hash info of this shop's script tags | ref.hash
#========================================================================================
sub GetUnityScriptTags {
    my $self                 = shift;
    my ($Processor,$aParams) = @_;

    my $Object = $aParams->[0];
    my $Site = $Object->getSite();

    my $Servlet;
    eval {
        $Servlet = GetAppServer()->currentServlet();
    };
    $Servlet = undef if $@;

    my $hScriptTagInfo;
    if ( $Servlet && $Servlet->can('intSessionID') ) {
        my $SessionValue = GetSessionAttribute($Servlet->intSessionID, "checkout-script-tags");
        $hScriptTagInfo = defined $SessionValue ? from_json($SessionValue) : undef;
    } else {
    }

    #-- add script tags to session if not already (prevents loading on every order process start)
    if( ref $hScriptTagInfo ne 'HASH' ) {
        my $ScriptTagAppClient = GetUnityScriptTagsApp($Site);
        my $ahRegisteredScriptTags = $ScriptTagAppClient->getEverySingleScriptTag();

        my $hShopContent;
        eval {
            $hShopContent = $Site->sendUnityRequest('GET', '/shop');
        };
        $hShopContent = undef if $@;

        #-- You know, this information will almost never change once a shop is set up, maybe we can cache the damn thing rather
        #-- than hit the j server again and again ...
        my $hInfo = {
            'ScriptTags' => $ahRegisteredScriptTags,
            'CookieNotice' => 1, #-- Assume it is active, in case we can't get the info from ep-ui
            'IsCookieFirstInstalled' => 0, #-- Assume it is NOT installed
            'GTMID' => undef,
        };

        if ( defined $hShopContent ) {
            $hInfo->{'CookieNotice'} = $hShopContent->{'userSettings'}->{'cookieNotice'}->{'active'};

            #-- Be consistent with ep-ui, parse the code for the id
            #-- epages-ui\packages\fusion-frontend\src\web\utils\googleTagManager.ts
            #-- epages-ui\packages\fusion-frontend\src\middleware\reactRouter.js
            my $GTMRegEx = '<script[\s\S]+gtm\.start[\s\S]+https:\/\/[\w.]+\/gtm\.js[\s\S]+(GTM-[A-Z0-9]{1,7})[\s\S]+<\/script>';
            my $GTMHeadHtml = $hShopContent->{'userSettings'}->{'googleTagManager'}->{'headHtml'};
            my $IsActive = $hShopContent->{'userSettings'}->{'googleTagManager'}->{'active'};
            if ( $IsActive && $GTMHeadHtml =~ m/$GTMRegEx/ ) {
                $hInfo->{'GTMID'} = $1;
            }
        }

        if ( $Servlet && $Servlet->can('intSessionID') ) {
            my $AppsClient = DE_EPAGES::AppStore::API::Object::AppsClient->create(
                {
                    Shop   => $Site,
                    Locale => $Servlet->locale,
                }
            );
            my $ahMyApps = $AppsClient->myApps();

            #-- We don't really want to hardcode the app id of the cookiefirst app and a regex is a bit flaky, but ... any other suggestions?
            #detailId 5E8A000D-164D-4474-8FE3-0349DF7B9AE2
            #name CookieFirst Cookie-Zustimmungsverwaltung
            foreach my $hApp ( @$ahMyApps ) {
                if ( $hApp->{'name'} =~ m/cookiefirst/i ) {
                    $hInfo->{'IsCookieFirstInstalled'} = 1;
                    last;
                }
            }

            $hScriptTagInfo = $hInfo;
            SetSessionAttribute( $Servlet->intSessionID, 'checkout-script-tags', to_json($hScriptTagInfo) );
        }
    }

    return $hScriptTagInfo;
}

#========================================================================================
# function     IS_APP_AVAILABLE
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsAppAvailable = IS_APP_AVAILABLE( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# description  Finds out whether an app is avilable or not.
#               #APP_ID(#Shop.ID, appUrl, $appType)
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       $IsAppAvailable | Whether app is installed or not | boolean
#========================================================================================
sub IS_APP_AVAILABLE {
    my $self                 = shift;
    my ($Processor,$aParams) = @_;
    my ($Shop,$Url,$Type)    = $self->_getShopUrlType($aParams);

    return DE_EPAGES::AppStore::API::Tools->new->IsAppAvailable($Shop,$Url,$Type);
}

#========================================================================================
# function     APP_ID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AppID = APP_ID( $Processor, $aParams );
#----------------------------------------------------------------------------------------
# description  Get the ID of an App.
#               #APP_ID(#Shop.ID, appUrl, $appType)
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $aParams | List of Parameters | array ref
# return       $AppID | Whether app is installed or not | boolean
#========================================================================================
sub APP_ID {
    my $self                 = shift;
    my ($Processor,$aParams) = @_;
    my ($Shop,$Url,$Type)    = $self->_getShopUrlType($aParams);

    return DE_EPAGES::AppStore::API::Tools->new->AppID($Shop,$Url,$Type);
}

#---------------------------------------------------------------------------
# Private Methods
#---------------------------------------------------------------------------
sub _getShopUrlType {
   my $self                        = shift;
   my ($aParams)                   = @_;
   my ($objectID,$appUrl,$appType) = @$aParams;
   my $Shop                        = _LoadObject($objectID);

   return ($Shop,$appUrl,$appType);
}

#---------------------------------------------------------------------------
# Wrappers
#---------------------------------------------------------------------------
sub _LoadObject {
    my ($objectID) = @_;
    return LoadObject($objectID);
}


1;
