#========================================================================================
# §package      DE_EPAGES::PayPalPPCP::UI::PaymentMethodPayPalPPCP
# §base         DE_EPAGES::Payment::UI::PaymentMethod
# §state        public
#----------------------------------------------------------------------------------------
# §description  UI subs for the PaymentMethod PayPalPPCP
#========================================================================================
package DE_EPAGES::PayPalPPCP::UI::PaymentMethodPayPalPPCP;
use base DE_EPAGES::Payment::UI::PaymentMethod;

use strict;
use JSON;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::PayPalPPCP::API::Constants qw ( $MERCHANT_INFO_REFRESH_THRESHOLD_MINS @DISABLEABLE_FUNDING_SOURCES $DISABLEABLE_FUNDING_SOURCES_MAP );

#========================================================================================
# §function     Save
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->Save( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Save the settings on the settings page
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub Save {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Form = $Servlet->form;

    if ( $Form->value('SaveFundingSourceChoices') ) {
        my $hJSONInfo = {};

        foreach ( @DISABLEABLE_FUNDING_SOURCES ) {
            $hJSONInfo->{$_} = $Form->value('EnableFS_' . $_) ? 1 : 0;
        }

        $PaymentMethod->set({
            'FundingSourceChoices' => $hJSONInfo,
        });

    }
    return $self->SUPER::Save($Servlet);
}

#========================================================================================
# §function     ViewSettings
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Settings page for a ppcp payment method
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Shop = $PaymentMethod->getSite();

    if ( $PaymentMethod->hasAppCredentials() ) {

        my $MerchantInfoWasSet = 0;

        my $hMerchantInfoJSON = $PaymentMethod->get('MerchantInfoJSON');
        if ( ! $hMerchantInfoJSON ) {
            $PaymentMethod->setMerchantInfo();
            $MerchantInfoWasSet = 1;
        } else {
            my $PrimaryEmailConfirmed = $hMerchantInfoJSON->{'primary_email_confirmed'};
            my $PaymentsReceivable = $hMerchantInfoJSON->{'payments_receivable'};
            if ( ! $PrimaryEmailConfirmed || ! $PaymentsReceivable ) {
                $PaymentMethod->setMerchantInfo();
                $MerchantInfoWasSet = 1;
            }
        }

        #-- If merchant info was not called, let's call it, provided that the refresh 'threshold' has elapsed
        #-- This information can change if they onboard for another product outside of ePages, for e.g. adding PUI after onboarding for just PUI
        my $LastUpdatedEpoch = $hMerchantInfoJSON->{'epages_last_updated'};
        my $NowEpoch = GetCurrentDateTime()->epoch();
        my $Difference = $NowEpoch - $LastUpdatedEpoch;
        if ( $Difference > $MERCHANT_INFO_REFRESH_THRESHOLD_MINS * 60 ) {
            $PaymentMethod->setMerchantInfo();
        }

        my $RegisteredWebHooksResponse = $PaymentMethod->get('RegisteredWebHooksResponse');
        #-- No webhook response stored, that means our onboarding attempt failed? Or for whatever reason . . .
        #-- And the app credentials we have stored should only be used by us, so unregister all webhooks, and re-register them
        if ( ! $RegisteredWebHooksResponse ) {
            my $Unregistered = $PaymentMethod->unregisterAppWebHooks();

            if ( ! $Unregistered ) {
                #-- Error

            } else {
                my $Registered = $PaymentMethod->registerOnBoardWebHooks();
                if ( ! $Registered ) {
                    
                } else {
                    $PaymentMethod->handleScriptTags($Shop, 0);
                }
            }
        }
    } else {
        #-- Double check to see if we can SandBox, used to ensure that rare situations like sandbox credentials being removed when we are already in test mode and thus need to reset this value to 0
        $PaymentMethod->set({
            'IsInTestMode' => 0,
        }) if ! $PaymentMethod->canSandBox();

        my $ReferralLink = $PaymentMethod->getPartnerReferralLink();

        if ( $ReferralLink ) {
            $Servlet->vars( 'ReferralLink', $ReferralLink );
        }
    }

    $Servlet->vars( 'CountrySupportsPayLater', $PaymentMethod->countrySupportsPayLater() );

    #-- Initialise the funding sources array if not already defined
    #-- Add any funding sources not already saved so that they can be enabled by default
    my $hFundingSourceChoices = $PaymentMethod->get('FundingSourceChoices');
    $hFundingSourceChoices = {} if ref $hFundingSourceChoices ne 'HASH';
    foreach ( @DISABLEABLE_FUNDING_SOURCES ) {
        $hFundingSourceChoices->{$_} = 1 if ! exists $hFundingSourceChoices->{$_};
    }
    $PaymentMethod->set({
        'FundingSourceChoices' => $hFundingSourceChoices,
    });
    $Servlet->vars( 'DISABLEABLE_FUNDING_SOURCES', \@DISABLEABLE_FUNDING_SOURCES );
    $Servlet->vars( 'DISABLEABLE_FUNDING_SOURCES_MAP', $DISABLEABLE_FUNDING_SOURCES_MAP );

}

#========================================================================================
# §function     CompleteOnBoard
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->CompleteOnBoard( $Servlet );
#----------------------------------------------------------------------------------------
# §description  Called when a merchant finishes onboarding
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub CompleteOnBoard {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Shop = $PaymentMethod->getSite();
    my $Form = $Servlet->form;

    #-- If we already have credentials stored, do not allow this, since the payment method should be 'disconnected' before onboarding can happen again
    return if $PaymentMethod->hasAppCredentials();

    my $AuthCode = $Form->value('AuthCode');
    my $SharedID = $Form->value('SharedID');

    return unless $AuthCode && $SharedID;

    my $TempAccessToken = $PaymentMethod->getMerchantAccessTokenWithAuthCodeSharedID($AuthCode, $SharedID);
    $Form->executeFormError({
        'Name'       => 'CompleteOnBoard',
        'Reason'     => 'NoAccessToken',
    }) unless $TempAccessToken;

    my $hClientApp = $PaymentMethod->getMerchantApp($TempAccessToken);
    my $ClientID = ref $hClientApp eq 'HASH' ? $hClientApp->{'client_id'} : undef;
    my $ClientSecret = ref $hClientApp eq 'HASH' ? $hClientApp->{'client_secret'} : undef;
    $Form->executeFormError({
        'Name'       => 'CompleteOnBoard',
        'Reason'     => 'NoMerchantApp',
    }) unless $ClientID && $ClientSecret;

    my $CountrySupportsPayLater = $PaymentMethod->countrySupportsPayLater();
    $PaymentMethod->set({
        'ClientID' => $ClientID,
        'ClientSecret' => $ClientSecret,
        'EnablePUI' => 0,                                               #-- Disable PUI/PayLater initially
        'EnablePayLater' => $CountrySupportsPayLater ? 1 : 0,           #-- PayPal insists $.$
        'EnablePayLaterBanners' => $CountrySupportsPayLater ? 1 : 0,    #-- PayPal insists $.$
        'EnableCartButtons' => 1,
    });

    $Servlet->vars('ONBOARD_JUST_COMPLETED', 1);

    #-- Register webhooks
    my $Success = $PaymentMethod->registerOnBoardWebHooks();
    if ( ! $Success ) {
        $PaymentMethod->handleScriptTags($Shop, 1);
    } else {
        $PaymentMethod->handleScriptTags($Shop, 0);
    }
}

#========================================================================================
# §function     DisconnectPPCP
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $self->DisconnectPPCP( $Servlet );
#----------------------------------------------------------------------------------------
# §description  'Disconnect' a PayPal REST app, mainly just clear attributes and unregister webhooks
#----------------------------------------------------------------------------------------
# §input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::Permission
#========================================================================================
sub DisconnectPPCP {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my $PaymentMethod = $Servlet->object();
    my $Shop = $PaymentMethod->getSite();

    return unless $PaymentMethod->hasAppCredentials();

    my $Success = $PaymentMethod->disconnect();
    
    #-- Error if not successful
    $Form->executeFormError({
        'Name'       => 'DisconnectPPCP',
        'Reason'     => 'DisconnectFailed',
    }) unless $Success;

    if ($Success) {
        $PaymentMethod->handleScriptTags($Shop, 1);
        $PaymentMethod->set({
            'IsActivated' => 0,
        });
    }
}

1;
