#==============================================================================
# package      DE_EPAGES::Zettle::UI::ZettleEnvironment
# state        private
#==============================================================================
package DE_EPAGES::Zettle::UI::ZettleEnvironment;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use Digest::SHA qw( hmac_sha256_hex );
use JSON qw ( decode_json );

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog LogError );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Zettle::API::Constants qw ( ALLOWED_EXPORT_INTERVALS VALIDATION_TOKEN_KEY EXPORT_STATUS );
use DE_EPAGES::Zettle::API::Tools qw ( ValidateCallbackToken );
use DE_EPAGES::Zettle::API::ZettleService;
use DE_EPAGES::Zettle::API::ZettleUrlBuilder;
use DE_EPAGES::Zettle::UI::WebhookHelper qw ( SetupWebhooks );


#ERRORDEFINITION
sub ErrorDefinition { return {
        'CallbackSignatureMismatch' => 'Callback message signature mismatch for MessageID=#MessageID',
    }
}
#ERRORDEFINITION END

sub ViewZettleGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object();

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    my $InitialExportStatus = $ZettleService->getInitialExportStatus();
    $Servlet->vars('IsInitialExportCompleted', $InitialExportStatus == EXPORT_STATUS->{COMPLETED});

    #TODO: check WS state (refresh token expired or Zettle not reachable)
    $Servlet->vars('WebServiceError', 0);
    
    # Ensure required location UUIDs are setup if initial export is complete
    $self->_ensureLocationSetup($Environment) if $InitialExportStatus == EXPORT_STATUS->{COMPLETED};

    $self->SUPER::View($Servlet);

    return;
}

sub ViewZettleSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object();

    my $UrlBuilder = DE_EPAGES::Zettle::API::ZettleUrlBuilder->new();
    my $Shop = $Environment->getSite();
    my $ConnectUrl = $UrlBuilder->buildConnectUrl($Servlet, $Shop);
    $Servlet->vars('ConnectUrl', $ConnectUrl);
        
    my $AdminUrl = $UrlBuilder->buildAdminUrl();
    $Servlet->vars('ZettleAdminUrl', $AdminUrl);

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);

    my $InitialExportStatus = $ZettleService->getInitialExportStatus();

    $Servlet->vars('IsInitialExportCompleted', $InitialExportStatus == EXPORT_STATUS->{COMPLETED});
    $Servlet->vars('IsInitialExportRunning', $InitialExportStatus == EXPORT_STATUS->{RUNNING});

    #TODO: check WS state (refresh token expired or Zettle not reachable)
    $Servlet->vars('WebServiceError', 0);

    if ($InitialExportStatus == EXPORT_STATUS->{FAILED}) {
        $Servlet->vars('ZettleExportErrors', 1);
        my $InitialExportUUID = $Environment->get('InitialExportUUID');
        $Servlet->vars('GlobalExportUUID', $InitialExportUUID);
    }
    
    # Ensure required location UUIDs are setup if initial export is complete
    $self->_ensureLocationSetup($Environment) if $InitialExportStatus == EXPORT_STATUS->{COMPLETED};

    $self->SUPER::View($Servlet);

    return;
}

sub ZettleUpdateConfig {
    my $self = shift;
    my ($Servlet) = @_;
    
    my $Environment = $Servlet->object();
    my $Form = $Servlet->form();

    # Verify export checkboxes
    my $ProductExport = $Form->formValue('ProductExport');
    my $FormPriceExport = $Form->formValue('PriceExport');
    my $ExportProductPrices = 0;
    $ExportProductPrices = 1 if defined $FormPriceExport && $FormPriceExport eq 'on';

    # Retrieve ProductExportInterval when ProductScheduledExport is enabled
    my $ExportScheduleInterval = 0;
    if($ProductExport == 1){
        $ExportScheduleInterval = $Form->formValue('ExportScheduleInterval');
        return unless $ExportScheduleInterval ~~ ALLOWED_EXPORT_INTERVALS;
    }

    $Environment->set({
        'ExportScheduleInterval' => $ExportScheduleInterval,
        'ExportProductPrices'   => $ExportProductPrices,
    });

    return;
}

#========================================================================================
# function     ZettleDisconnect
# state        public
#----------------------------------------------------------------------------------------
# syntax       ZettleDisconnect($Servlet);
#----------------------------------------------------------------------------------------
# description  Trigger disconnect for Zettle
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ZettleDisconnect {
    my $self = shift;
    my ($Servlet) = @_;
    
    my $Environment = $Servlet->object;
    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    #Check if metchant wants also to remove the product from Zettle
    my $RemoveExportedProductsFromZettle = $Servlet->form->value('RemoveExportedProductsFromZettle') eq 'true';

    $ZettleService->disconnect($RemoveExportedProductsFromZettle);

    $Servlet->vars( 'ContentType', 'text/x-json' );
    my $hData = {
        result => 'OK'
    };
    $Servlet->writeContentOutput($hData);
    $Servlet->vars( 'ExitAfterEvent', 1 );
    $Servlet->vars( 'ResponseContentProcessed', 1 );

    return;
}

sub ExportProductUpdates {
    my $self = shift;
    my ($Servlet) = @_;
    
    my $Environment = $Servlet->object;

    # Run product export (asynchronous process)
    my $hData;
    eval {
        $Environment->exportProductUpdates({'IncludePrices' => 0});
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $hData->{success} = 0;
        $hData->{GUID} = $Error->guid;
    } else {
        $hData->{success} = 1;
    }

    my $LastProductExport = $Environment->lastProductExport();
    my $Shop = $Environment->getSite();
    $hData->{lastupdate} = $self->_formatLastUpdate($LastProductExport, $Shop);

    $Servlet->vars( 'ContentType', 'text/x-json' );
    $Servlet->writeContentOutput($hData);
    $Servlet->vars( 'ExitAfterEvent', 1 );
    $Servlet->vars( 'ResponseContentProcessed', 1 );
}

sub ExportPriceUpdates {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object;

    # Run price export (asynchronous process)
    my $hData;
    eval {
        $Environment->exportPriceUpdates();
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $hData->{success} = 0;
        $hData->{GUID} = $Error->guid;
    } else {
        $hData->{success} = 1;
    }

    my $LastPriceExport = $Environment->lastPriceExport();
    my $Shop = $Environment->getSite();
    $hData->{lastupdate} = $self->_formatLastUpdate($LastPriceExport, $Shop);

    $Servlet->vars( 'ContentType', 'text/x-json' );
    $Servlet->writeContentOutput($hData);
    $Servlet->vars( 'ExitAfterEvent', 1 );
    $Servlet->vars( 'ResponseContentProcessed', 1 );
}

sub ZettleInitialExport {
    my $self = shift;
    my ($Servlet) = @_;
    
    my $Environment = $Servlet->object;
    my $Form = $Servlet->form;

    # Read form settings
    my $ExportSettings;
    foreach my $ZettleAttribute (qw(ReplaceZettleLibrary ExportProductPrices)) {
        $ExportSettings->{$ZettleAttribute} = $Form->value($ZettleAttribute) ? 1 : 0;
    }

    # Update initial export settings in Zettle environment
    $Environment->set($ExportSettings);
    
    # Run initial product export (asynchronous process)
    my $hData;
    eval {
        my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
        my $ReplaceZettleLibrary = $ExportSettings->{'ReplaceZettleLibrary'};
        my $ExportPrices = $ExportSettings->{'ExportProductPrices'};
        $ZettleService->initialExport($ReplaceZettleLibrary, $ExportPrices);
        
        # Setup Zettle webhooks to receive incoming updates
        SetupWebhooks($Servlet, $Environment);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $Servlet->vars('ZettleExportErrors', 1);
        my $InitialExportUUID = $Environment->get('InitialExportUUID');
        $hData->{GlobalExportUUID} = $InitialExportUUID;
        $hData->{success} = 0;
    }else{
        $Servlet->vars('IsInitialExportCompleted', 0);
        $Servlet->vars('IsInitialExportRunning', 1);
        $hData->{success} = 1;
    }
    
    $Servlet->vars( 'ContentType', 'text/x-json' );
    $Servlet->writeContentOutput($hData);
    $Servlet->vars( 'ExitAfterEvent', 1 );
    $Servlet->vars( 'ResponseContentProcessed', 1 );
}

#========================================================================================
# function     ZettleStockCallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       ZettleStockCallback( $Servlet );
#----------------------------------------------------------------------------------------
# description  Handles InventoryBalanceChanged webhook event via asynchronous updates.
#               As stated in Zettle doc (https://github.com/iZettle/api-documentation/blob/master/pusher-api/api-reference.md#supported-events)
#               webhook is triggered when:
#               <ul>
#                   <li>A sale is made for a product and purchase gets created.
#                   <li>The stock of a product gets updated in the product library.
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ZettleStockCallback {
    my $self = shift;
    my ($Servlet) = @_;
    my $MethodName = (caller(0))[3];

    my $Environment = $Servlet->object();
    my $IncomingRequest = $Servlet->request;
    my $EventContent = $IncomingRequest->content;

    my $Logger = DE_EPAGES::Zettle::API::Logger->new();
    $Logger->logCommunication($MethodName.' [incoming event]', $IncomingRequest);
    
    eval {
        # Validate callback token
        $self->_authenticateCallback($Servlet);

        # Validate message signature
        my $hWebhookEvent = decode_json($EventContent);

        # Ignore test request: this request is sent by Zettle to test callback URL when a subscription is created
        if($hWebhookEvent->{'eventName'} eq 'TestMessage'){
            GetLog->debug("$MethodName - ignoring ZettleStockCallback test request");
            return;
        }

        my $MessageSignature = $Servlet->requestHeaders('x-izettle-signature');
        $self->_verifySignature($MessageSignature, $hWebhookEvent, $Environment);

        my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
        $ZettleService->incomingStockUpdate($hWebhookEvent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
    }
}


#========================================================================================


# Retrieve ValidationToken from Servlet params and ensure it's a valid callback token
sub _authenticateCallback {
    my $self = shift;
    my ($Servlet) = @_;

    my $Environment = $Servlet->object();
    my $Token = $Servlet->params(VALIDATION_TOKEN_KEY);

    ValidateCallbackToken($Token, $Environment);
}

#========================================================================================
# function     _verifySignature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_verifySignature($MessageSignature, $hWebhookEvent, $Environment);
#----------------------------------------------------------------------------------------
# description  Use ZettleEnvironment InventorySubscriptionSigningKey to verify x-izettle-signature header as described in Zettle documentation.
#               See: https://github.com/iZettle/api-documentation/blob/master/pusher-api/user-guides/create-subscriptions.md#step-4-set-up-verification-for-events-origin
#----------------------------------------------------------------------------------------
# input        $MessageSignature | x-izettle-signature header retrieved from servlet request | String
# input        $hWebhookEvent | decoded content of an incoming webhook event.
#                                See <DE_EPAGES::Zettle::API::WebService::DTO::WebhookEventDTO> for expected fields | ref.hash
# input        $Environment | a ZettleEnvironment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#----------------------------------------------------------------------------------------
# throws       CallbackSignatureMismatch
#========================================================================================
sub _verifySignature {
    my $self = shift;
    my ($MessageSignature, $hWebhookEvent, $Environment) = @_;
    my $SigningKey = $Environment->getInventorySubscriptionSigningKey();
    my $MessageID = $hWebhookEvent->{'messageId'};
    my $Timestamp = $hWebhookEvent->{'timestamp'};
    my $Payload = $hWebhookEvent->{'payload'};

    my $Content = join('.', $Timestamp, $Payload);
    my $CalculatedSignature = hmac_sha256_hex($Content, $SigningKey);
    Error('CallbackSignatureMismatch',
            {
                'MessageID' => $MessageID,
                'SigningKey' => $SigningKey,
                'Content' => $Content,
                'CalculatedSignature' => $CalculatedSignature,
                'ExpectedSignature' => $MessageSignature
            }) unless ($CalculatedSignature eq $MessageSignature);
}

#========================================================================================
# function     _formatLastUpdate
# state        private
#----------------------------------------------------------------------------------------
# syntax       $datestring = $self->_formatLastUpdate($LastUpdate, $Shop);
#----------------------------------------------------------------------------------------
# description  Parse given datetime and extract a string in the same format as [datetime] TLE formatter.
#               See <function DE_EPAGES::Core::API::Object::DateTimeFormatter::format_datetime>.
#----------------------------------------------------------------------------------------
# input        $LastUpdate | a datetime object | datetime
# input        $Shop | a Shop object | DE_EPAGES::Shop::API::Object::Shop
#----------------------------------------------------------------------------------------
# return       A string representation for the given datetime according to Shop locale.
#========================================================================================
sub _formatLastUpdate {
    my $self = shift;
    my ($LastUpdate, $Shop) = @_;

    my $LocaleID = $Shop->get('LocaleID');
    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new(
        'locale' => $LocaleID,
    );

    return $Formatter->format_datetime($LastUpdate, 'datetime');
}

#========================================================================================
# function     _ensureLocationSetup
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ensureLocationSetup($Environment);
#----------------------------------------------------------------------------------------
# description  Check if required location configuration is complete.
#               Update missing location UUIDs.
#----------------------------------------------------------------------------------------
# input        $Environment | environment object | DE_EPAGES::Zettle::API::Object::ZettleEnvironment
#========================================================================================
sub _ensureLocationSetup {
    my $self = shift;
    my ($Environment) = @_;

    return if $Environment->isLocationDataSetup();

    eval {        
        $Environment->updateLocationData();
    };
    if(ExistsError()){ LogError(GetError()); }
}

1;
